package com.tuoheng.admin.service.third.airport.execute;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.tuoheng.admin.entity.Inspection;
import com.tuoheng.admin.entity.InspectionHistory;
import com.tuoheng.admin.enums.InspectionStatusEnum;
import com.tuoheng.admin.enums.code.inspection.ExecuteInspectionCodeEnum;
import com.tuoheng.admin.mapper.InspectionHistoryMapper;
import com.tuoheng.admin.mapper.InspectionMapper;
import com.tuoheng.admin.mapper.TenantMapper;
import com.tuoheng.admin.service.third.dsp.IDspService;
import com.tuoheng.admin.tzhl.config.TZHLConfig;
import com.tuoheng.admin.tzhl.constant.TZHLConstant;
import com.tuoheng.admin.tzhl.response.TZHLAirportLineResponse;
import com.tuoheng.admin.tzhl.response.TZHLOnlineUavResponse;
import com.tuoheng.admin.tzhl.response.TZHLShelterTaskResponse;
import com.tuoheng.admin.tzhl.service.airportline.TZHLGetAirportLineListService;
import com.tuoheng.admin.tzhl.service.fly.onlineuav.OnlineUavService;
import com.tuoheng.admin.tzhl.service.task.ShelterTaskService;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.HttpUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class ExecuteTaskService {

    @Autowired
    private TenantMapper tenantMapper;

    @Autowired
    private InspectionMapper inspectionMapper;

    @Autowired
    private InspectionHistoryMapper inspectionHistoryMapper;

    @Autowired
    private TZHLGetAirportLineListService tzhlGetAirportLineListService;

    @Autowired
    private ShelterTaskService shelterTaskService;

    public JsonResult executeTask(String userId, Inspection inspection) {
        log.info("进入泰州海陵区城管，执行任务，inspectionId={}", inspection.getId());

        TZHLAirportLineResponse tzhlAirportLineResponse = tzhlGetAirportLineListService.getAirportLineById(inspection.getInspectionLine());
        if (ObjectUtil.isEmpty(tzhlAirportLineResponse)) {
            log.info("泰州海陵区城管，执行任务，航线不存在，airportLineId:{}", tzhlAirportLineResponse.getId());
            return JsonResult.success("泰州海陵区城管，航线不存在");
        }

        Long recordId = shelterTaskService.distributeTask(inspection, tzhlAirportLineResponse);
        if (ObjectUtil.isEmpty(recordId)) {
            log.info("泰州海陵区城管，执行任务，下发任务失败，status={}", inspection.getStatus());

            inspection.setStatus(InspectionStatusEnum.FLIGHT_FAILED.getCode());
            this.updateInspection(userId, inspection, recordId);

            this.insertInspectionHistory(inspection, "下发任务失败");

            return JsonResult.success("泰州海陵区城管，执行任务，下发任务失败");
        }

        inspection.setStatus(InspectionStatusEnum.PREPARING.getCode());
        this.updateInspection(userId, inspection, recordId);

        log.info("任务下发成功，修改任务状态，status={}", inspection.getStatus());

        return JsonResult.success();
    }

    /**
     * 修改任务执行
     *
     * @param userId
     * @param inspection
     */
    private void updateInspection(String userId, Inspection inspection, Long recordId) {
        inspection.setExecutionStatus(2);   // 修改任务执行状态，防止手动执行和定时重复
        inspection.setStatus(inspection.getStatus());
        inspection.setUpdateUser(userId);
        inspection.setUpdateTime(DateUtils.now());
        inspection.setRecordId(recordId);
        inspectionMapper.update(inspection);
    }

    /**
     * 更新任务失败记录
     *
     * @param inspection
     */
    private void insertInspectionHistory(Inspection inspection, String msg) {
        InspectionHistory inspectionHistory = new InspectionHistory();
        inspectionHistory.setTenantId(inspection.getTenantId());
        inspectionHistory.setInspectionId(inspection.getId());
        inspectionHistory.setHistoryName(msg);
        inspectionHistory.setCreateUser("");
        inspectionHistory.setCreateTime(DateUtils.now());
        inspectionHistoryMapper.insert(inspectionHistory);
    }
}