package com.tuoheng.admin.tzhl.service.fly.log;

import cn.hutool.core.util.ObjectUtil;
import com.tuoheng.admin.entity.Inspection;
import com.tuoheng.admin.enums.InspectionStatusEnum;
import com.tuoheng.admin.mapper.InspectionMapper;
import com.tuoheng.admin.service.third.dsp.IDspService;
import com.tuoheng.admin.tzhl.response.TZHLFlyLogResponse;
import com.tuoheng.admin.tzhl.response.TZHLOnlineUavResponse;
import com.tuoheng.admin.tzhl.service.fly.onlineuav.OnlineUavService;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.JsonResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class FlyLogBySuccessService {

    @Autowired
    private InspectionMapper inspectionMapper;

    @Autowired
    private OnlineUavService onlineUavService;

    @Autowired
    private IDspService dspService;

    public void getFlyLog(Inspection inspection, TZHLFlyLogResponse flyLogResponse) {
        log.info("获取飞行日志，status=success，title={}", flyLogResponse.getTitle());
        if (InspectionStatusEnum.IN_FLIGHT.getCode() == inspection.getStatus()) {
            log.info("获取飞行日志，该任务状态已经改是飞行中，inspectionid={}", inspection.getId());
            return;
        }
        log.info("获取飞行日志，status=success，title={}", flyLogResponse.getTitle());
        inspection.setStatus(InspectionStatusEnum.IN_FLIGHT.getCode());
        inspection.setUpdateUser("");
        inspection.setUpdateTime(DateUtils.now());
        inspectionMapper.updateStatus(inspection);

        log.info("获取飞行日志，status=success，修改任务状态，status={}", inspection.getStatus());

        TZHLOnlineUavResponse tzhlOnlineUavResponse = onlineUavService.getOnlineUav();
        if (ObjectUtil.isEmpty(tzhlOnlineUavResponse)) {
            log.info("泰州海陵区城管，执行任务，获取推流地址失败");
            return;
        }

        // 通知DSP
        JsonResult result = dspService.startAI(inspection, tzhlOnlineUavResponse.getPushUrl(), tzhlOnlineUavResponse.getPushUrl());
        if (0 != result.getCode()) {
            log.info("泰州海陵区城管，执行任务，通知DSP失败");
            inspection.setStatus(InspectionStatusEnum.FLIGHT_FAILED.getCode());
        }
    }

}
